/**************************************************************************************************
* NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
*
* Copyright 2010 Flexera Software, Inc. and/or InstallShield Co. Inc.  All Rights Reserved.  
*
* This computer program is the property of Flexera Software, Inc. and InstallShield Co. Inc. of 
* Schaumburg, Illinois, U.S.A.  Any use, copy, publication, distribution, display, modification, 
* or transmission of this computer program in whole or in part in any form or by any means without
* the prior express written permission of Flexera Software, Inc. or InstallShield Co. Inc. is 
* strictly prohibited.
* 
* Except when expressly provided by Flexera Software, Inc. or InstallShield Co. Inc. in writing,
* possession of this computer program shall not be construed to confer any license or rights under 
* any of Flexera Software, Inc.s or InstallShield Co. Inc.s intellectual property rights, whether
* by estoppel, implication, or otherwise.  
* 
* ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
**************************************************************************************************/
/*
	This header file is part of a sample utility that demonstrates Composite Transactions.

	It declares the FlxCtuContext (context) object and the functions that relate to it.
	It is the top level object and contains all other objects that persist for the execution
	of the utility. 
*/
#ifndef COMP_TRAN_UTIL_CONTEXT_H
#define COMP_TRAN_UTIL_CONTEXT_H

#include "FlxActTransaction.h"
#include "compTranUtilCommandDef.h"
#include "compTranUtilCommand.h"

/* One of these is returned to the OS when the utility exits. */
typedef enum flxCtuExitCode_enum
{
	flxCtuExitSuccess					= 0,
	flxCtuExitInputError				= 1,	/* Invalid command line input.					*/
	flxCtuExitInitError					= 2,	/* FNP initialisation failed.					*/
	flxCtuExitCommsError				= 3,	/* Error communicating with server.				*/
	flxCtuExitResponseProcessingError	= 4,	/* flxActTransactionProcessResponse failed.		*/
	flxCtuExitApiError					= 5,	/* Other FNP API call failed.					*/
	flxCtuExitFileError					= 6,	/* Error accessing file.						*/
	flxCtuExitMemoryError				= 7,	/* No memory.									*/
	flxCtuExitAssert					= 8,	/* An assert failed.							*/

	flxCtuExitUnknown					= 9,
} FlxCtuExitCode;


typedef enum flxCtuParameterType_enum
{
	flxCtuParameterTypeCommand,
	flxCtuParameterTypeValue,
	flxCtuParameterTypeKeyValue,
} FlxCtuParameterType;

typedef struct FlxCtuContext_struct 
{
	const char*					pszProgramName;
	FlxCtuExitCode				exitCode;
	FlxActBool					bIsServer;
	FlxActBool					bIsVerbose;
	FlxActBool					bCommonLibraryInitialised;
	FlxActHandle				hApi;
	FlxActTransaction			hTransaction;

	const FlxCtuCommandDef*		pCommandDefs;
	size_t						commandDefCount;
	
	const FlxCtuCommand**		ppCommands;
	size_t						commandCount;

	const FlxCtuCommand*		pMainCommand;
	FlxActBool					bIsMainCommandSet;

	size_t						dictionaryCount;
	const FlxCtuDictionary**	ppDictionaries;
} FlxCtuContext;


void flxCtuContextCreate(const FlxCtuCommandDef*	pCommandDefs, 
						 size_t						commandDefCount,
						 FlxCtuContext**			ppContext);

void flxCtuContextDestroy(FlxCtuContext* pContext);

FlxActBool flxCtuContextHasCommand(const FlxCtuContext* pContext, const char* pszCommandName);

FlxActBool flxCtuContextGetCommandByName(const FlxCtuContext*	pContext, 
									     const char*			pszCommandName, 
									     const FlxCtuCommand**	ppCommand);

size_t flxCtuContextGetCommandCount(const FlxCtuContext*	pContext);

const FlxCtuCommand* flxCtuContextGetCommand(const FlxCtuContext* pContext, size_t commandIndex);

FlxActBool flxCtuContextFindCommandDef(const FlxCtuContext*		pContext, 
									   const char*				pszComamndOrAlias,
									   const FlxCtuCommandDef**	ppCommandDef);

FlxActBool flxCtuContextValidateCommands(FlxCtuContext* pContext);

/*
	Load a file into memory (free with flxCtuFree).
*/
FlxActBool flxCtuContextFileLoad(FlxCtuContext*	pContext, 
								 const char*	pszFilename, 
								 char**			ppFileContent,
								 size_t*		pContentSize);

/*
	Load file stream content into memory (free with flxCtuFree).
*/
FlxActBool flxCtuContextStreamLoad(FlxCtuContext*	pContext, 
								 FILE*			pFileStream,
								 const char*	pszFilename, 
								 char**			ppFileContent,
								 size_t*		pContentSize);

/*
	Create/replace a file from a buffer.
*/
FlxActBool flxCtuContextFileDump(FlxCtuContext*	pContext, 
								 const char*	pszFilename, 
								 const char*	pFileContent,
								 size_t			contentSize);

void flxCtuContextPrintLastApiError(FlxCtuContext* pContext);

void flxCtuContextPrintApiFlexError(FlxCtuContext* pContext, FlxActError* pFlxError);
void flxCtuContextPrintLastApiFlexError(FlxCtuContext* pContext);

const char* flxCtuGetTextForResult(FlxActTranResult result);

/*
	Macro for one-line error handling for API errors that indicate a developer issue, 
	for example an invalid handle or parameter.
	The target function must be implemented by users of this module.
*/
void flxCtuOnUnexpectedApiError(FlxCtuContext* pContext, const char* pszFilename, int line);
#define FLX_CTU_EXIT_UNEXPECTED_API_ERROR(pContext)		\
	flxCtuOnUnexpectedApiError(pContext, __FILE__, __LINE__);
	


#endif	/* Include guard */
